
%% Data Reading

clear all

% read data from csv file:

data = csvread('trade_data.csv',1,0);

N = sqrt(size(data,1))  %number of countries
T = N
K = size(data,2)-4      %number of regressors

% define NxN matrix of outcomes Y, and KxNxN multi-matrix of regressors X:
Y0 = reshape(data(:,3),N,N);
Y1 = exp(Y0).*(Y0>0); 
Y  = Y1/std(Y1(:));
X = zeros(K,N,N);
for k=1:K
  X(k,:,:) = reshape(data(:,4+k),N,N);
end

% drop first regressor:
X(1,:,:)=[];
X(10,:,:) = [];
X(8,:,:) = [];
K=K-3

% redefine island and landlock regressor:

%X(K-2,:,:)=( squeeze(X(K-2,:,:))==2 );   %island is dummy for island == 2 (i.e. both islands)
%X(K,:,:)=( squeeze(X(K,:,:))==2 );       %landlock is dummy for landlock == 2 (i.e. both landlocked)

% check which regressors are binary (just for information at this point)
binary_regressor_list=[];
for k=1:K
  if X(k,:,:) == (X(k,:,:)==1)
    binary_regressor_list = [  binary_regressor_list, k];  
  end  
end    
binary_regressor_list

% add a constant to the model:
%X(K+1,:,:)=ones(N,N);
%K=K+1;

% Estimate panel poisson with factors:
lambda_known = ones(N,1);           %known factor loading, corresponding to time dummies
f_known      = ones(N,1);           %known factors, corresponding to individual dummies
Rex1         = size(lambda_known,2)
Rex2         = size(f_known,2)
weight       = ones(N,N)-eye(N);    %all diagonal observations are missing, so get zero weight

precision_beta = 10^-7;
repMIN         = 25;
repMAX         = 26;
MAX_STEPS      = 250;

R=0                   %number of additional factors
dist='Poisson';            %logit or probit or poisson 

%
% Initial Values
% (i) Based on Orthogonal Polynomials of 1:T 
xt  = [1:T]';
One = ones(T,1);
X0  = xt;
if R>1
    for i = 2: R
       X0 = [X0 xt.^i];
    end
end

Xtt = One;
for i=1:size(X0,2)
  Xtt =[Xtt X0(:,i)-Xtt*pinv(Xtt)*X0(:,i)];
end
fhat = Xtt(:,2:(R+1)); 

% (ii) Based on Principal Components 
% Obtain initial values by linear principal components 
[fhat2, D] = eig(Y'*Y);
fhatN      = fhat2(:,1:R)*sqrt(T);

% Estimation: without bias correction
[beta,alpha,gamma,lambda,f,exitflag,obj,APE,Var_beta,Var_APE] = FactorMLE(dist, Y, X, weight, lambda_known, f_known, R, precision_beta, repMIN, repMAX, MAX_STEPS);

beta
%return
%%%%% REDEFINE OBJECTIVE:
obj=obj*N*N / sum(sum(weight));

% Report: without bias correction
save(['SAVE_APPL_RES_' dist '_K=' num2str(K) '_R=' num2str(R) '_Rex1=' num2str(Rex1) '_Rex2=' num2str(Rex2) '.mat']);
format long
obj
beta
std_beta=sqrt(diag(Var_beta))
APE
std_APE=sqrt(diag(Var_APE))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% WRITE CSV FILE:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

outputfile=['SAVE_APPL_NobiasCorrection_RES_' dist '_K=' num2str(K) '_R=' num2str(R) '_Rex1=' num2str(Rex1) '_Rex2=' num2str(Rex2) '.csv'];
disp(['Write results into file: ' outputfile])  
  
%open output file
fid = fopen(outputfile,'wt+');

%write parameters:
fprintf(fid, 'Results from Empirical Trade Application \n\n');
fprintf(fid, 'N, %d\n', N);
fprintf(fid, 'K, %d\n', K);
fprintf(fid, 'R, %d\n', R);
fprintf(fid, 'Rex1, %d\n', Rex1);
fprintf(fid, 'Rex2, %d\n', Rex2);
fprintf(fid, '\n');
fprintf(fid, 'objective = , %f\n', obj);
fprintf(fid, '\n\n\n');

%write header:
fprintf(fid, 'beta (std), APE (std) \n\n');
for k=1:K
    fprintf(fid, '%.4f, %.4f', beta(k), APE(k));
    fprintf(fid, '\n');
    fprintf(fid, '''(%.4f)'', ''(%.4f)''' , std_beta(k) , std_APE(k));
    fprintf(fid, '\n');
end
fprintf(fid, '\n\n\n');

for k=1:K
fprintf(fid, '%.3f, %.3f', beta(k), APE(k));
    fprintf(fid, '\n');
    fprintf(fid, '''(%.3f)'', ''(%.3f)''' , std_beta(k) , std_APE(k));
    fprintf(fid, '\n');
end   
fprintf(fid, '\n\n\n');
  
for k=1:K
    fprintf(fid, '%.2f, %.2f', beta(k), APE(k));
    fprintf(fid, '\n');
    fprintf(fid, '''(%.2f)'', ''(%.2f)''' , std_beta(k) , std_APE(k));
    fprintf(fid, '\n');
end   
  
fclose(fid);  

  
  
 
